from urllib.parse import quote
import re
import networkx as nx
import matplotlib.pyplot as plt
from lxml.html import fromstring, tostring
from .enums import *
from .config import ResourceConfig
from .search_retrieve import Record
from .diagnostics import Diagnostic, DiagnosticTypes


class AnnisResponseParser:
    """
    Parses responses from an ANNIS instance.
    """

    def __init__(self):
        self.pc = None      # POS convertor, rebuilt with each parse call

    def process_hits(self, hits, config: ResourceConfig, searchOptions: dict,
                     diagnostics: list[Diagnostic], advancedHits=False):
        """
        Process hits returned by ANNIS, which take the form of
        graphML strings.
        If anything goes wrong, add Diagnostic objects to diagnostics list.
        Return a list of Record objects.
        """
        records = []
        for iHit in range(len(hits)):
            try:
                hits[iHit] = nx.parse_graphml(hits[iHit].decode('utf-8'))
            except:
                raise Diagnostic(DiagnosticTypes.sru, 1,
                                 details='Could not parse graphML data returned by the ANNIS instance.')

        for hit in hits:
            # for node, data in hit.nodes(data=True):
            #     print(node, data)
            roots = [n for n, d in hit.in_degree() if d == 0]
            textSegments = []
            nodeData = {
                node[0]: node[1] for node in hit.nodes(data=True)
            }
            for root in roots:
                usedNodes = set()
                textSegment = ''
                for e in nx.bfs_edges(hit, root):
                    for side in (0, 1):
                        if e[side] in usedNodes:
                            continue
                        usedNodes.add(e[side])
                        node = nodeData[e[side]]
                        tok = ''
                        tier = ''
                        for k in node.keys():
                            if k == 'annis::tok':
                                tok = node[k]
                            elif k.endswith('Gloss'):
                                tier = 'Gloss'
                        if tier == 'Gloss':
                            textSegment += tok + ' '
                if len(textSegment) > 0:
                    textSegments.append(textSegment.strip())
            records.append(' <...> '.join(textSegments))
        return records

    def parse(self, responseData, config: ResourceConfig, searchOptions: dict):
        """
        Read graphML response with the first N hits returned by an ANNIS
        instance. The hits to process are stored in responseData['hits'].
        Return a list of Record objects and the total number of
        records found.
        """
        diagnostics = []
        advancedHits = False
        dataViewsRequested = {v.strip() for v in searchOptions['x-fcs-dataviews'].split(',') if len(v.strip()) > 0}
        if 'adv' in dataViewsRequested:
            advancedHits = True
        nRecords = responseData['n_hits']
        records = []
        if searchOptions['startRecord'] > 1 and nRecords < searchOptions['startRecord']:
            # We don't actually care about startRecord, but we should
            # return a fatal diagnostic if it is larger than the number
            # of hits.
            diagnostics.append(Diagnostic(DiagnosticTypes.sru, 61))
            return records, nRecords, diagnostics
        if len(responseData['hits']) <= 0:
            nRecords = 0
        else:
            try:
                records = self.process_hits(responseData['hits'], config, searchOptions,
                                            diagnostics, advancedHits=advancedHits)
            except Diagnostic as d:
                diagnostics.append(d)
        return records, nRecords, diagnostics


if __name__ == '__main__':
    pass

